/**
 * Change the way the Atoms look like in the JTree
 */
package core;

import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.TreeCellRenderer;
import java.awt.Component;
import objects.*;
import javax.swing.*;
import java.awt.Color;
import java.awt.GridLayout;

/**
 * 
 * @see Rendering
 * @see RenderList
 * @see javax.swing.JTree.TreeCellRenderer
 */
public class AJTreeCellRenderer implements TreeCellRenderer {
	JLabel titleLabel;
	JPanel renderer;
	DefaultTreeCellRenderer defaultRenderer = new DefaultTreeCellRenderer();
	Color backgroundSelectionColor, backgroundNonSelectionColor;
	
	/**
	 * standard Constructor
	 *
	 */
	public AJTreeCellRenderer() {
		renderer = new JPanel(new GridLayout(0, 1));
		titleLabel = new JLabel(" ");
		titleLabel.setForeground(Color.blue);
		renderer.add(titleLabel);

		backgroundSelectionColor = defaultRenderer
				.getBackgroundSelectionColor();
		backgroundNonSelectionColor = defaultRenderer
				.getBackgroundNonSelectionColor();

	}

	/**
	 * defines the view of each cell
	 * @param	tree	defines the tree to use
	 * @param	value	the object to render
	 * @param	selected	the state of each entry
	 * @param	expanded	the expandstat
	 * @param	leaf		is allready a leaf
	 * @param	row			row number (depends on the whole tree)
	 * @param	hasFocus	focus of each cell
	 * 
	 */
	public Component getTreeCellRendererComponent(JTree tree, Object value,
			boolean selected, boolean expanded, boolean leaf, int row,
			boolean hasFocus) {
		Component returnValue = null;
		if ((value != null) && (value instanceof DefaultMutableTreeNode)) {
			Object userObject = ((DefaultMutableTreeNode) value)
					.getUserObject();
			if (userObject instanceof Atom) {
				Atom a = (Atom) userObject;
				titleLabel.setText(row + " " + a.toString());

				if (selected) {
					renderer.setBackground(backgroundSelectionColor);
				} else {
					renderer.setBackground(backgroundNonSelectionColor);
				}
				renderer.setEnabled(tree.isEnabled());
				returnValue = renderer;
				return returnValue;
			}
		}
		if (returnValue == null) {
			returnValue = defaultRenderer.getTreeCellRendererComponent(tree,
					value, selected, expanded, leaf, row, hasFocus);
		}
		return returnValue;
	}
}
