/**
 * 
 * Adds Objects to Renderinglist
 * 
 */
package core;

import javax.management.DescriptorKey;
import javax.media.opengl.GL;
import javax.media.opengl.GLAutoDrawable;
import javax.xml.ws.ServiceMode;

import objects.Atom;
import objects.Dodekaeder;
import objects.FileASCII;
import objects.FilePDB;
import objects.Surface;
import objects.FileXML;
import objects.BuckyBall;

/**
 * 
 * @see	Rendering
 * @see RenderList
 * @version 0.5
 */

public class AddToRendering extends Rendering {
	private static final long serialVersionUID = 1L;
	private FileXML testXML;
	
	
	public AddToRendering() {	}
	
	/**
	 * @see 			core.Rendering#init(javax.media.opengl.GLAutoDrawable)
	 * adds the startup XML file to Rendering List
	 */
	public void init(GLAutoDrawable drawable) {
		super.init(drawable);
		addFileXML();	
	}
	
	/**
	 * adds a standard Dodecahedron to the Rendering List 
	 * @see			model.Dodekaeder
	 */
	public void addDode(){
		Dodekaeder tmp = new Dodekaeder();
		list.add(tmp);
	}
	
	/**
	 * adds a standard BuckyBall to the Rendering List 
	 * @see			model.BuckyBall
	 */
	public void addBuckyBall(){
		BuckyBall tmp = new BuckyBall();
		list.add(tmp);
	}
	
	/**
	 * adds a standard BuckyBall to the Rendering List 
	 * @see			model.BuckyBall
	 * @param		file	path to the File to load
	 */
	public void addFileASCII(String file){
		FileASCII tmp = new FileASCII(file);
		list.add(tmp);
	}
	
	/**
	 * adds coordinates from a PDB file to the Rendering List | Only for testing purposal
	 * @see			model.FilePDB
	 */
	@Deprecated
	public void addFilePDB(){
		FilePDB tmp = new FilePDB();
		list.add(tmp);
	}
	
	/**
	 * adds coordinates from a PDB file to the Rendering List
	 * @see			model.FilePDB
	 * @param		file	path to the File to load
	 */
	public void addFilePDB(String file){
		FilePDB tmp = new FilePDB(file);
		list.add(tmp);
	}
	
	/**
	 * adds a Surface to Rendering List
	 * @see			model.Surface
	 * @param		x 	width
	 * @param		y 	height
	 * @param		z	length
	 */
	public void addSurface(int x, int y, int z){
		Surface tmp = new Surface(x, y, z);
		list.add(tmp);
	}
	
	/**
	 * adds a standard Surface to Rendering List
	 * @see			model.Surface()
	 * @deprecated	only for testing purposal
	 */
	@Deprecated
	public void addSurface(){
		Surface tmp = new Surface();
		list.add(tmp);
	}
	
	/**
	 * loads a SML file and refresh the JTree 
	 * @see			model.FileXML#FileXML(String)
	 * @param		file	path to the File to load
	 */
	public void addFileXML(String file){
		testXML = new FileXML(list, file);
		if (testXML.workspaceSetting())
			background = testXML.getBackground();
		list.refreshJTree();
	}
	
	/**
	 * loads the startup SML file 
	 * @see			model.FileXML#FileXML(String)
	 */
	public void addFileXML(){
		addFileXML("model/start.sml");
	}
	
	/**
	 * gives direct access to the Rendering List
	 * @see			core.RenderList()
	 */
	public RenderList getList(){
		return list;
	}
	
	public void addFileTGA(){
		objects.HeightMap h = new objects.HeightMap();
		list.add(h);
	}

	
	
	
	
}
