/**
 * This Class is the Graphical User interface for object settings.
 * Objects can be Mols or Atoms. 
 * 
 * 
 */
package core;

import javax.swing.JFrame;
import javax.swing.JPanel;
import java.awt.BorderLayout;
import javax.swing.JScrollPane;
import javax.swing.JLabel;
import javax.swing.JTextField;
import objects.Structure;
import objects.AtomNeighbours;
import javax.swing.JButton;
import javax.swing.JCheckBox;

/**
 * 
 * 
 */
public class ObjectSettings {
	private JFrame jFrame = null; // @jve:decl-index=0:visual-constraint="198,80"
	private JPanel jContentPane = null;
	private JScrollPane jScrollPane = null;
	private JLabel jLabel = null;
	private JPanel jPanel = null;
	protected Structure obj;
	private JPanel jPanel1 = null;
	private JButton jButton = null;
	private JButton jButton1 = null;
	protected JTextField T1 = null;
	protected JTextField T2 = null;
	protected JTextField T3 = null;
	protected JTextField T4 = null;
	protected JCheckBox C1 = null;
	private AtomNeighbours atom = null;
	private boolean isNew = false;
	private Structure saveTo = null;

	/**
	 * Constructor for Mol's.
	 * @param obj	a Mol in form of a Structure Object
	 */
	public ObjectSettings(Structure obj) {
		
		this.obj = obj;
	}

	
	/**
	 * Objectsettings for Atoms
	 * 
	 * @param obj		Object which contains the Atom
	 * @param atom		The Atom which is part of Object
	 */
	public ObjectSettings(Structure obj, AtomNeighbours atom) {
		
		this.obj = obj;
		this.atom = atom;
	}

	/**
	 * A single Atom. Settings are not full compatible. Its just for some special needs
	 * Without a belonging Mol its not useful.
	 * @param atom
	 */
	public ObjectSettings(AtomNeighbours atom) {
		this.atom = atom;
	}

	/**
	 * Constructor for new Atoms or Structures to change
	 * The need of this function is to add new Atoms to a Molecule
	 * @param obj	Molecule which the new Atom belongs to
	 * @param newOne	faked parameter just to make it accessilbe
	 */
	public ObjectSettings(Structure obj, boolean newOne) {
		this.atom = new AtomNeighbours();
		this.saveTo = obj;
		isNew = true;
	}

	/**
	 * This method initializes jFrame
	 * 
	 * @return javax.swing.JFrame
	 */
	public JFrame getJFrame() {
		if (jFrame == null) {
			jFrame = new JFrame();
			jFrame.setSize(new java.awt.Dimension(310, 163));
			jFrame.setContentPane(getJContentPane());

		}
		return jFrame;
	}

	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			jLabel = new JLabel();
			jLabel.setText("Object settings");
			jContentPane = new JPanel();
			jContentPane.setLayout(new BorderLayout());
			jContentPane.add(getJScrollPane(), java.awt.BorderLayout.CENTER);
			jContentPane.add(jLabel, java.awt.BorderLayout.NORTH);
			jContentPane.add(getJPanel1(), java.awt.BorderLayout.SOUTH);
		}
		return jContentPane;
	}

	/**
	 * This method initializes jScrollPane
	 * 
	 * @return javax.swing.JScrollPane
	 */
	private JScrollPane getJScrollPane() {
		if (jScrollPane == null) {
			jScrollPane = new JScrollPane();
			jScrollPane.setViewportView(getJPanel());
		}
		return jScrollPane;
	}

	/**
	 * This method initializes jPanel
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJPanel() {
		if (jPanel == null) {
			jPanel = new JPanel();
			jPanel.setLayout(new java.awt.GridLayout(0, 2));

			T1 = new JTextField();
			T2 = new JTextField();
			T3 = new JTextField();
			T4 = new JTextField();
			C1 = new JCheckBox();
			if (atom == null) {
				T4.setText(String.valueOf(obj.getZ()));
				T3.setText(String.valueOf(obj.getY()));
				T2.setText(String.valueOf(obj.getX()));
				T1.setText(String.valueOf(obj.getScale()));
				
			} else {
				T4.setText(String.valueOf(atom.getZ()));
				T3.setText(String.valueOf(atom.getY()));
				T2.setText(String.valueOf(atom.getX()));
				T1.setText(String.valueOf(0));
			}

			if (atom == null) {
				jPanel.add(new JLabel("Distance"));
				jPanel.add(T1);
				
			}

			jPanel.add(new JLabel("Pos X"));
			jPanel.add(T2);

			jPanel.add(new JLabel("Pos Y"));

			jPanel.add(T3);

			jPanel.add(new JLabel("Pos Z"));

			jPanel.add(T4);
			
			if(atom == null){
			
				jPanel.add(new JLabel("Draw connections"));
				
				jPanel.add(C1);
				C1.setSelected(obj.isDrawConnection());
			}


		}
		return jPanel;
	}

	/**
	 * This method initializes jPanel1
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJPanel1() {
		if (jPanel1 == null) {
			jPanel1 = new JPanel();
			jPanel1.add(getJButton(), null);
			jPanel1.add(getJButton1(), null);
		}
		return jPanel1;
	}

	/**
	 * This method initializes jButton
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getJButton() {
		if (jButton == null) {
			if (isNew)
				jButton = new JButton("Insert");
			else
				jButton = new JButton("Save");
			jButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					System.out.println("actionPerformed()"); 
						if (atom == null) {
							obj.setScale(Float.parseFloat(T1.getText()
									.trim()));
							obj.setX(Double.parseDouble(T2.getText().trim()));
							obj.setY(Double.parseDouble(T3.getText().trim()));
							obj.setZ(Double.parseDouble(T4.getText().trim()));
							if(obj.isDrawConnection() != C1.isSelected()){
								obj.setDrawConnection(C1.isSelected());
							}
						} else {

							atom.setX(Double.parseDouble(T2.getText().trim()));
							atom.setY(Double.parseDouble(T3.getText().trim()));
							atom.setZ(Double.parseDouble(T4.getText().trim()));
							if(!isNew)
								obj.reinit();
						}
					if(isNew){
						if (!saveTo.getAtome().contains(atom)) {
							saveTo.getAtome().add(atom);
							getJButton().setText("Save");
						}
						saveTo.reinit();
					}

				}
			});
		}
		return jButton;
	}

	/**
	 * This method initializes jButton1
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getJButton1() {
		if (jButton1 == null) {
			jButton1 = new JButton("Exit");
			jButton1.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					System.out.println("actionPerformed()"); 
					jFrame.dispose();
				}
			});
		}
		return jButton1;
	}

}
