/**
 * Generic class which extends the Stack
 * This class makes it Possible to use the Memory only once for the Rentering Structure
 * and the JTree. No objected needs to be stored twice and every viewed objected has direct
 * effect on the scene
 */
package core;

import java.util.Stack;
import javax.swing.tree.DefaultMutableTreeNode;

import objects.Atom;
import objects.Structure;
import objects.AtomNeighbours;
import objects.WWManager;

import objects.nononjects.*;
/**
 * To use the Generic feature you need to extend this class for every type you want to use
 * 
 * @param <T>	Generic Parameter which need to be extend to different types if they dont
 * 		belong to the Main Structure object
 */
public class RenderList<T> extends Stack<T> {

	private DefaultMutableTreeNode treeNodes;
	private boolean refreshJTree = false;
	/**
	 * Main Constructor which doesnt instance any object
	 */
	public RenderList() {

	}

	/**
	 * Builds the minimal JTree leafs
	 * @return	the first leaf of the JTree
	 */
	public DefaultMutableTreeNode getJTreeNode() {
		if (treeNodes == null) {
			treeNodes = new DefaultMutableTreeNode("Project");

		}
		refreshJTree();

		return treeNodes;
	}

	
	
	/**
	 * Getter function for is JTree uptodate.
	 * @return the refreshJTree
	 */
	public boolean isRefreshJTree() {
		return refreshJTree;
	}

	/**
	 * Setter function for the JTree which marks the JTree to reupdate his leafs
	 * @param refreshJTree the refreshJTree to set
	 */
	public void setRefreshJTree(boolean refreshJTree) {
		this.refreshJTree = refreshJTree;
	}

	
	
	
	
	/**
	 * Return a new JTree which contains all Objects and every transactionvalue
	 * @return full JTree Node
	 */
	public DefaultMutableTreeNode refreshJTree() {
		treeNodes.removeAllChildren();
		int n = 0;
		for (T object : this) {
			n++;
			DefaultMutableTreeNode node = new DefaultMutableTreeNode(object);

			treeNodes.add(node);
			if (object instanceof Structure) {
				Structure myStruct = (Structure) object;
				for (int i = 0; i < myStruct.getAtome().size(); i++) {

					DefaultMutableTreeNode anode = new DefaultMutableTreeNode(myStruct.getAtome().get(i));

					Stack<AtomNeighbours> an = ((Structure) object).getAtome();
					AtomNeighbours neigh = an.get(i);
					for (AtomNeighbours a : neigh.getNeighbours()) {

						DefaultMutableTreeNode nnode = new DefaultMutableTreeNode(a);
						if (neigh.getNeighbourJJ(a) > 0) {
							nnode.add(new DefaultMutableTreeNode(neigh
									.getNeighbourJJ(a)));
						}
						anode.add(nnode);

						// anode.add(new DefaultMutableTreeNode(a));
					}

					node.add(anode);

				}
			}
		}

		WWManager ww = WWManager.getInstance();
		if (ww.getWW().size() > 0) {
			DefaultMutableTreeNode wwTreeNode = new DefaultMutableTreeNode(
					"Wechselwirkungen");
			int i = 0;
			for (WechselWirkung wwf : ww.getWW()) {
				i++;
				DefaultMutableTreeNode wwNode = new DefaultMutableTreeNode(wwf);
				
				Stack<AtomNeighbours> allReadyin = new Stack<AtomNeighbours>();
				
				for (T object : this) {
					DefaultMutableTreeNode tNode = null;
					if (object instanceof Structure) {
						for(AtomNeighbours a : ((Structure) object).getAtome()){
							if(a.getNeighbourCount() > 0){
								if (tNode == null){
									tNode = new DefaultMutableTreeNode(object);
								}
								for(AtomNeighbours neig : a.getNeighbours()){
									//TODO hier weiter schreiben
									// es werden bis jetzt noch alle einsortiert.
									//!allReadyin.contains(neig) && 
									if(a.getWechselwirkungsIndex(neig) == wwf.getIdentifier()){
										tNode.add(new DefaultMutableTreeNode(neig));
										allReadyin.add(neig);
									}
								}
							}
						}
					}
					if (tNode != null)
						wwNode.add(tNode);
				}
				
				wwTreeNode.add(wwNode);
			}
			treeNodes.add(wwTreeNode);
		}
		
		
		return treeNodes;
	}

	
	
	/**
	 * Set every atom selected depending by its transactionvalue Object.
	 * @param	wechsel	transaction object 
	 * @param	mode	enable or disable the the atoms which belongs to this group
	 * @see model.nonobjects.WechselWirkung
	 */
	public void setSelectedByWW(WechselWirkung wechsel, boolean mode){
		
		for(Atom a : wechsel.getConnected()){
			if(a != null){
				a.setSelected(mode);
			}
		}
		
		for(T obj : this){
			boolean con = false;
			if(obj instanceof Structure){
				
				for(Atom a : wechsel.getConnected()){
					((Structure) obj).getAtome().contains(a);
					con = true;
				}
				
			}
			if(con == true)
				((Structure) obj).reinit();
		}
	}
	
	
	/**
	 * Returns the XML (SML) code which repesents the scene by using each objects own toXML
	 * function.
	 * Attention: When every you wrote extrem big files this function gets really slow cause of the
	 * recursive function which are called by every object
	 * @return	Valid XML File which represents the scene
	 */
	public String toXML() {
		String buf = "<spin>\n" + "\t<workspace>\n";
		for (T obj : this) {
			if (obj instanceof Structure) {
				Structure struct = (Structure) obj;
				buf += "\t\t<mol " + struct.XMLAttribute() + ">\n";
				for (AtomNeighbours a : struct.getAtome()) {
					buf += "\t\t\t<atom " + a.XMLAttribute() + ">\n";
					int tmp = 0;
					for (AtomNeighbours n : a.getNeighbours()) {

						// TODO wenn nicht innerhalb der Struktur eigenen Fall
						// bauen
						// Strukturintern
						
						
						
						if(struct.getAtome().indexOf(n) > -1){
							buf += "\t\t\t\t<connect nn=\""
									+ (struct.getAtome().indexOf(n)+1) + "\"";
							if (a.getWechselwirkungsIndex(tmp) != 0) {
								buf += " jj=\"" + a.getWechselwirkungsIndex(tmp)
										+ "\"";
							}
	
							buf += "/>\n";
						} else {
							int objNo = -1;
							int objNumber = 0;
							for(T theObjs: this){
								if(theObjs instanceof Structure && objNo == -1){
									Structure cyle = ((Structure) theObjs);
									if(cyle.getAtome().contains(n)){
										System.out.println("Found @ "+objNumber);
										objNo = objNumber;
									}
									
								}
								objNumber++;
							}
							if(objNo > -1){
								Structure found = (Structure) this.get(objNo);
								int iatom = (found.getAtome().indexOf(n)+1);
								buf += "\t\t\t\t<connect nn=\""+iatom+"@"+(objNo+1)+ "\"";
								
								if (a.getWechselwirkungsIndex(tmp) != 0) {
									buf += " jj=\"" + a.getWechselwirkungsIndex(tmp)
											+ "\"";
								}
		
								buf += "/>\n";
								
							} else
								System.err.println(this.getClass()+":: Error, cant find : "+tmp);
						}
						tmp++;
					}
					buf += "\t\t\t</atom>\n";
				}
				buf += "\t\t</mol>\n";

			}

			WWManager ww = WWManager.getInstance();
			if (ww.getWW().size() > 0) {
				for(WechselWirkung w : ww.getWW())
					buf += "\t\t<ww number=\""+w.getIdentifier()+"\" value=\""+w.getValue()+"\"/>\n";
			}
			
			
		}

		buf += "\t</workspace>\n" + "</spin>\n";
		return buf;
	}

	/**
	 * Return a String which represents the JJ file.
	 * @return valid JJ file
	 */
	public String getJJ(){
		return getDesc("JJ");
	}
	
	/**
	 * Returns a String which represents the NN file (next neighbours) to work with
	 * spinheage@Home
	 * @return 	valid string which represents the content for the NN file
	 */
	public String getNN(){
		return getDesc("NN");
	}
	
	/**
	 * Returns the transactionvalue. This String can be stored to work with 
	 * Spinheage@Home
	 * @return	String which represents WW file content
	 */
	public String getWW(){
		WWManager wm = WWManager.getInstance();
		String ret = "";
		for (WechselWirkung ww : wm.getWW()) {
			ret += ww.getValue()+"\n";
		}
		return ret;
	}
	
	
	/**
	 * the basic function behind getNN() and getJJ
	 * @param select	can be [NN|JJ]
	 * @return	Valid content for the file 
	 */
	protected String getDesc(String select){
		int maxNN = 0;
		
		String ret = "";
		WWManager wwMan = WWManager.getInstance();
		for(AtomNeighbours atom : linearModel()){
			if(maxNN < atom.getNeighbours().size())
					maxNN = atom.getNeighbours().size();
		}
		
		ret += maxNN+"\n";
		
		//anpassung an orginal 
		int size  = linearModel().size()+1;
		
		for(AtomNeighbours atom : linearModel()){
			for (AtomNeighbours neigh : atom.getNeighbours()) {
				if(select.equals("NN"))
					ret += (size-linearModel().search(neigh))+" ";
				
				else if(select.equals("JJ"))
					ret += atom.getNeighbourJJ(neigh)+" ";
			}
			
			ret += "\n"; 
		}
		
		return ret;
	}
	
	/**
	 * A simplification of the molecular structures. This reorders the main model and drops 
	 * all Structure information
	 * Attention: This function only returns atoms which have a connection to a different atom!
	 * @return	returns all objects which have a connection to a different atom
	 */
	private Stack<AtomNeighbours> linearModel(){
		Stack<AtomNeighbours> linear = new Stack<AtomNeighbours>();
		for (T obj : this) {
			if (obj instanceof Structure) {
				for(AtomNeighbours atom : ((Structure) obj).getAtome()){
					if(atom.getNeighbours().size() > 0)
					linear.add(atom);
				}
			}
		}
		return linear;
	}
	
}
