/**
 * Basic Atom Model
 */
package objects;

import java.awt.event.MouseEvent;

import javax.media.opengl.*;
import javax.media.opengl.glu.*;

/**
 * Instance a Atom which can be used to draw or modify
 * memory for the core Object will be stored only once.
 */

public class Atom extends Object3D {
	private boolean selected = false;
	static protected boolean roundCords = true; 
	private GLUquadric qobj0;
	static protected GLU glu = new GLU();
	public double wcoord[] = new double[4];	
	static protected int glListed= 0;
	
	/**
	 * Public Constructor without parameters.
	 * Default values of 0 0 0  will be used.
	 *
	 */
	public Atom(){
		init();
	}
	
	/**
	 * Public Constructor with parameters which sets the position.
	 * All settings are relative to the molecule position (Mol defines the new 0 0 0 ) for
	 * each atom.
	 * @param x		defines the X values in the coordinate system 
	 * @param y		defines the Y values in the coordinate system 
	 * @param z		defines the Z values in the coordinate system
	 */
	public Atom(double x, double y, double z) {
		super(x, y, z);
		init();
		
	}

	/**
	 * Builds the Object settings.
	 * The init method will be automatically called whenever you extend this class.
	 * If you dont want to have precompiled Objects you need to "overload" this method
	 * and handle object generation by yourself.
	 */
	private void init(){
		GLU glu = new GLU();
		if (glListed == 0) {
			
			qobj0 = glu.gluNewQuadric();	
			GL gl = glu.getCurrentGL();
			glListed = gl.glGenLists(1);
			gl.glNewList(glListed, GL.GL_COMPILE);
			
			
			
			 float mat_ambient[] = {0.24f, 0.19f, 0.07f, 1.0f};
			 float mat_diffuse[] = {0.75f, 0.60f, 0.22f, 1.0f};
			 float mat_specular[] = {0.55f, 0.55f, 0.55f, 1.0f};
			 float mat_shininess = 51.2f;

			 //gl.glMaterialfv(GL.GL_FRONT, GL.GL_AMBIENT, mat_ambient,0 );
			 //gl.glMaterialfv(GL.GL_FRONT, GL.GL_DIFFUSE, mat_diffuse,0);
			 gl.glMaterialfv(GL.GL_FRONT, GL.GL_SPECULAR, mat_specular,0);
			 gl.glMaterialf(GL.GL_FRONT, GL.GL_SHININESS, mat_shininess);
			
			
			
			
				glu.gluQuadricDrawStyle(qobj0, GLU.GLU_FILL);
		        glu.gluQuadricNormals(qobj0, GLU.GLU_SMOOTH);
		    	glu.gluSphere(qobj0, 0.4f, 20, 20);
		    	glu.gluDeleteQuadric(qobj0);
		    	
				
			
			gl.glEndList();
		}
		
	}



	/**
	 * Marks the Atom to be selected. This has effect on how it will be rendered.
	 * You need to re-init the main object to see changes.
	 * @see core.Rendering
	 * @param value	[true|false]
	 */
	public void setSelected(boolean value) {
		this.selected = value;
	}

	/**
	 * Gettermethod for selected state.
	 * @return	select state
	 */
	public boolean isSelected() {
		return this.selected;
	}

	/**
	 * Draws the Object.
	 * 
	 */
	public void draw ()
   {
		GL gl = glu.getCurrentGL();
		gl.glPushMatrix();
		
			gl.glTranslatef((float)x, (float) y, (float) z);
			
			gl.glCallList(glListed);
		
		
		gl.glPopMatrix();
	
   }

	
	/*
	 * TODO this method needs to be rewritten and moved to the object generation (init)
	 */
	/**
	 * 
	 * This methode is not really used.
	 * Deprecaded method to draw the atom with a scaled possition.
	 * This part shouldnt be used.
	 * 
	 */
	public void draw(float scale){
		GL gl = glu.getCurrentGL();
		gl.glPushMatrix();
		
		gl.glTranslatef((float)x*scale, (float) y*scale, (float) z*scale);
		
		gl.glCallList(glListed);
		gl.glPopMatrix();
	}
	
	/**
	 * Return the World Coordinates of the Atom with the default scale.
	 * @return World Coords of the Atom (absolute possition)
	 */
	public double[] getWorldCords(){
		return getWorldCords(1.0f);
	}
	
	/**
	 * Return the World Coordinates of the Atom with the selectable scale.
	 * @param scale	scaling of the objects possition parameters
	 * @return	World Coords of the Atom (absolute possition) with chosen scale
	 */
	public double[] getWorldCords(float scale){
		GL gl = glu.getCurrentGL();
		int viewport[] = new int[4];
		double mvmatrix[] = new double[16];
		double projmatrix[] = new double[16];
		    
	
		  

	 	gl.glGetIntegerv(GL.GL_VIEWPORT, viewport, 0);
	    gl.glGetDoublev(GL.GL_MODELVIEW_MATRIX, mvmatrix, 0);
	    gl.glGetDoublev(GL.GL_PROJECTION_MATRIX, projmatrix, 0);
	    /* note viewport[3] is height of window in pixels */

	    
	    //setting it to gluProject	    
	    glu.gluProject((double) x*scale, (double) y*scale, (double) z*scale, //
	              mvmatrix, 0,
	              projmatrix, 0, 
	              viewport, 0, 
	              wcoord, 0);
		return wcoord;
	}
	
	/**
	 * Simplified coords of the Atom as String.
	 * @return coords of the Atom rounded
	 */
	public String toString(){
		if (roundCords)
			return "x : " + Math.round(x) + " y: " +  Math.round(y) + " z : " + Math.round(z);
		else 
			return "x : " + x + " y: " +  y + " z : " + z;
	}
	
	


}