/**
 * 
 */
package objects;

import java.util.Stack;
import java.io.*;
/**
 * 
 * This class works like an interface.
 * You have to overload the "readFile" function where you descripe the syntax of your 
 * File format.
 * 
 * This class still call the generate method from the Structure class.
 * if you dont want to use it, you simply just can ignore this one
 * but whenever you want to generate a file and get additional informations by file
 * you have to overload "readFile" and generate.
 * 
 * @see objects.Structure
 * 
 *
 */
public class File extends Structure {

	/**
	 * 
	 * @param atome	Atoms which are preconfigure
	 * @see objects.Structure
	 */
	public File(Stack atome) {
		super(atome);
		
		readFile();
		center();
		
	}

	/**
	 * 
	 * @param val	draw a connection between neighbours
	 * @param atom	atomes which represents the structure
	 */
	public File(boolean val, Stack atom){
		super(val, atom);
	}
	
	/**
	 * Default constructor.
	 */
	public File() {
		
		readFile();
		center();
		
	}
	
	/**
	 * 
	 * @param file File which fits to your syntax you definded in "readFile"
	 * 
	 */
	public File(String file){
		
		readFile(file);
	}
	
	/**
	 * 
	 * @param d draw connection between each neighbour
	 */
	public File(boolean d){
		super(d);
		readFile();
		center();
		
	}
	
	/**
	 * 
	 * Centers all atoms around the 0 0 0 point.
	 * Its using a simple measurment which just builds 
	 * SUM(AtomePositions) and substract it from the others.
	 * There can be strange effects whenever your structure is not balanced.
	 *
	 */
	protected void center(){
		Atom sum = new Atom(0,0,0);
		int i = 0;
		for (AtomNeighbours atom: atome) {
			sum.x += atom.getX();
			sum.y += atom.getY();
			sum.z += atom.getZ();
			i++;
		}
		
		for (AtomNeighbours atom: atome) {
			atom.x -= sum.x / i;
			atom.y -= sum.y / i;
			atom.z -= sum.z / i;
		}
		
		
		
		
	}
	
	/**
	 * Dummy function for the interface.
	 * Attention:
	 * !You HAVE TO overload this function!
	 * @return false whenever not overloaded
	 */
	protected boolean readFile(){
		return false;
	}
	
	/**
	 * Dummy function for the interface.
	 * Attention:
	 * !You HAVE TO overload this function!
	 * @return false whenever not overloaded
	 */
	public boolean readFile(String file){
		
		return false;
	}

	/**
	 * Reads a Neighbourfile which is written in the SPIN format.
	 * @param f	File which points to the SPIN file
	 * @return file correctly formated and reading ended without errors
	 */
	protected boolean readNeighbours(java.io.File f){
	
		try {
			
			FileInputStream fstream = new FileInputStream(f);
			DataInputStream in = new DataInputStream(fstream);
	
			int i = 0;
			
			while (in.available() != 0) {
			
				String tmp[] = in.readLine().split(" +");
				
				if(tmp.length > 0 && i > 0)
					for (String string : tmp) {
						int ato = Integer.parseInt(string)-1;
						if(ato > -1)
							atome.get(i-1).setNeighbours(atome.get(ato));
					} 
				i++;
				
				
			}
	
			in.close();
			return true;
		} catch (Exception e) {
			System.err.println("File input error");
			e.printStackTrace();
			//System.exit(1);
			return false;
		}
		
		
	}
	
	
	
	
	
	
}
