/**
 * 
 */
package objects;

import java.io.DataInputStream;
import java.io.FileInputStream;
import java.util.Stack;

/**
 * Loads a ASCII file format (the simples format which can used to display 3D Date)
 * Values are seperated by spaces (" ") and atoms are seperated by "new line"
 * if there are missing values 0 will be used
 * @see objects.File 
 */
public class FileASCII extends File {

	private java.io.File fileex = null;
	
	/**
	 * @param atome	Atoms which representates the Structure
	 * @see objects.File
	 */
	public FileASCII(Stack atome) {
		super(atome);
	
	}

	/**
	 * default constructor
	 * drawing of lines by default disabled
	 */
	public FileASCII() {
		super(false);
	
	}

	/**
	 * Read a specified file
	 * @param file	file which fits in readFile defined Format
	 */
	public FileASCII(String file){
		super(file);
	}
	
	/**
	 * Draw connection between neighbours
	 * @param d	draw a connection
	 */
	public FileASCII(boolean d) {
		super(d);
	
	}

	/**
	 * overloaded Method which contains all specs for the SAF format
	 * @see objects.File#readFile()
	 */
	public boolean readFile(String file) {
		
		fileex = new java.io.File(file);
		
		try {
			// Open the file that is the first
			// command line parameter
			FileInputStream fstream = new FileInputStream(file);

			// Convert our input stream to a
			// DataInputStream
			DataInputStream in = new DataInputStream(fstream);

			// Continue to read lines while
			// there are still some left to read

			while (in.available() != 0) {
				// Print file line to screen
				String tmp[] = in.readLine().split(" +");

				// skip header info
				if (debug)
					System.out.println("Found Atomnode");
				// ignore Atome
				atome.add(new AtomNeighbours(Double.parseDouble(tmp[0]), Double
						.parseDouble(tmp[1]), Double.parseDouble(tmp[2])));

			}

			in.close();
		} catch (Exception e) {
			System.err.println("File input error");
			e.printStackTrace();
			System.exit(1);
		}
		
		checkForFiles();
		
		return true;
	}
	
	/**
	 * This checks for different files which have additional Date
	 * in case of the SPIN project this can be Neighbours and the depending transactionvalues
	 */
	public void checkForFiles() {
		if(checkFiles("nn")){
			if(this.readNeighbours(new java.io.File(fileex.toString().substring(0, (fileex.toString().length()-3))+"nn"))){
				if(checkFiles("jj") && checkFiles("ww")){
					System.out.println("All the data you need");
				} else {
					System.out.println("jj: "+checkFiles("jj"));
					System.out.println("ww: "+checkFiles("ww"));
				}
					
			} else {
				System.err.println("Failed to read file");
			}
		}
			
	}
	
	
	/**
	 * a simple name converter which checks for the local filename and replace the extension
	 * @param ext	the file extionen to search for
	 * @return	if there is a file with this file extension it returns true
	 */
	protected boolean checkFiles(String ext){
		String orgi = fileex.getAbsolutePath();
		//System.out.println(orgi);
		String tmp = orgi.substring(0, (orgi.length()-3))+ext;
		System.out.println("Probing: "+fileex+" -> "+tmp);
		java.io.File test = new java.io.File(tmp);
		return test.isFile();
	}

}
