/**
 * 
 */
package objects;

import java.util.Stack;
import core.*;
import java.util.ArrayList;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import org.xml.sax.SAXException;
import org.w3c.dom.*;
import java.io.*;

/**
 * 
 * Reads the SML format for opening complex scenes
 * Cause of the Structure of this Classes the SML file needs to parsed into Steps
 * 
 * First: Get all Structures and Atoms
 * Seconds: Connections between each Atom
 * 
 * Otherwise its not possible to link connections between non existing Atoms if you generate them later
 * 
 * The FileXML needs direct access to the RenderingList to add more Structures at once
 * 
 * @see core.RenderList
 * 
 */
public class FileXML{

	private String myURI;

	private Document doc;

	private ArrayList<Node> nodeList;

	private ArrayList<Structure> slist;

	private Stack<AtomNeighbours> atombuf = new Stack<AtomNeighbours>();

	private AtomNeighbours lastAtom = null;

	private int atomAnzahl = 0;
	private int molAnzahl = 0;
	private RenderList rlist;
	
	private boolean workspaceChanged;
	
	private float[] background = {0,0,0,0};
	
	private float scale = 1.0f;
	
	private Object3D camPos = new Object3D();

	static private boolean debug = false;
	
	//default settings
	private String lastClass= "Structure";
	private String color;
	private Atom copyAtom;
	/**
	 * 
	 * @param list RenderList to add the Structures
	 * @see core.RenderList
	 */
	public FileXML(RenderList list) {
		super();
		init(list);
	}
	
	/**
	 * @param list RenderList to add the Structures
	 * @param url	file to load (SML format)
	 */
	public FileXML(RenderList list, String url) {
		super();
		init(list, url);
	}
	/**
	 * Never Uses public 
	 * this Function is for developers purposal only
	 * @param list	RenderList
	 * @see	core.RenderList
	 */
	private void init(RenderList list){
		init(list, null);
	}

	/**
	 * 
	 * @param list
	 * @param url
	 * @see model.FileXML#init
	 */
	private void init(RenderList list, String url){
		rlist = list;
		this.slist = new ArrayList<Structure>();
		workspaceChanged = false;
		if(url != null)
			readFile(url);
		else
			readFile();
		if(debug) System.out.println("atombuf size: "+ atombuf.size() +"|"+ slist.size());
		generate();
		
	}
	
	/**
	 * 
	 * Adds all Structures which are found to RenderList
	 * @see core.RenderList
	 */
	private void generate(){
		for (Structure object : slist) {
			rlist.add(object);
			if (debug)System.out.println("========= ADDED =========");
		}
	}
	
	/**
	 * reads the SML file and starts the parsing process
	 * @param url	absolute file path to the SML file
	 * @return	whenever no crash happens this will exit with success
	 */
	protected boolean readFile(String url){
		myURI = url;
		ArrayList files = new ArrayList<String>();
		nodeList = new ArrayList<Node>();
		Parsing();
		return true;
	}
	
	/**
	 * Developbers default xml file 
	 * @return true
	 */
	public boolean readFile() {
		return readFile("model/test.xml");
		
		
	}
	
	/**
	 * Starts the real parsing process by using the Java Utility for this.
	 * Mailformed XML files will throw a exception message 
	 *
	 */
	protected void Parsing() {
		DocumentBuilder parser;
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		factory.setValidating(false);
		factory.setNamespaceAware(false);

		try {
			parser = factory.newDocumentBuilder();
			doc = parser.parse(myURI);
			doc.getFirstChild();
			//System.out.println(doc.getFirstChild().getNodeName());
			//hier individueller parse
			
			getElementByName("spin");
			System.out.println(secondParse(doc.getFirstChild()));
			
			

		} catch (SAXException e) {
			// SAX-Fehler behandeln
			System.out.println(e.getMessage());
		} catch (IOException e) {
			// Fehler beim Lesen des URI behandeln
			System.out.println(e.getMessage());
		} catch (ParserConfigurationException e) {
			// Fehler behandeln, die auftreten, wenn die Factory
			// die angegebene Parser-Implementation nicht laden kann
			System.out.println(e.getMessage());
		}

	}

	/**
	 * recursive walking through the XML nodes
	 * @param node	the next node to walk through
	 * @return	the XML file in a reinterpreted form
	 */
	protected String getElement(Node node) {
		String buf = new String();
		if (node == null)
			return "";

		/*
		 * 
		 * Ignoriert text zwischen TAG's
		 * 
		 */

		if (node.getNodeName().equals("#text"))
			return "";

		

		NodeList subNodes = node.getChildNodes();
		// }
		if (!node.getNodeName().trim().equals("#text")
				&& !node.getNodeName().equals("spin"))
			nodeList.add(node);
		buf = "<" + node.getNodeName();// +"="+node.getNodeValue()+"
		// ("+subNodes.getLength()+")\n";

		if (node.hasAttributes()) {
			NamedNodeMap attributes = node.getAttributes();
			buf += " ";
			String tmpName = new String("");
			int tmpSize = 0;
			for (int i = 0; i < attributes.getLength(); i++) {

				if (attributes.item(i).getNodeName() == "size") {
					tmpSize = Integer.parseInt(attributes.item(i)
							.getNodeValue());
				}
				if (attributes.item(i).getNodeName() == "name") {
					tmpName = attributes.item(i).getNodeValue();
				}

				buf += attributes.item(i).getNodeName();
				buf += "=\"" + attributes.item(i).getNodeValue() + "\" ";
			}
			
		}
		buf += ">\n";

		for (int i = 0; i < subNodes.getLength(); i++) {
			Node subNode = subNodes.item(i);
			if (subNode.getNodeName().equals("#text")) {
				if (subNode.getNodeValue().trim().equals(""))
					buf += "\n\t" + getElement(subNode);
				else {
					System.out.println("+++++++" + subNode.getNodeValue());
				}
			} else {
				buf += "\n\t" + getElement(subNode);
			}
		}

		
		
		if (node.getNodeName() == "atom") {
			try {
				if (atombuf == null) {
					atombuf = new Stack<AtomNeighbours>();
					atomAnzahl = 0;
				}

				if (atombuf.size() == atomAnzahl
						|| atombuf.get(atomAnzahl) == null) {

					System.out.println("direct adding"+(atombuf.size()+1));
					AtomNeighbours a = new AtomNeighbours();
					
				
					
					
						atombuf.add(atomAnzahl, a);
				} else {
					System.out.println("one ignored");
				}
				lastAtom = atombuf.get(atomAnzahl);

				
				
				if (node.hasAttributes()) {
					NamedNodeMap attributes = node.getAttributes();
					for (int i = 0; i < attributes.getLength(); i++) {
						
						
						if (attributes.item(i).getNodeName() == "xyz") {
							setXYZ(attributes.item(i).getNodeValue());
						} else
						if (attributes.item(i).getNodeName() == "x") {
							lastAtom.setX(Double.parseDouble(attributes.item(i).getNodeValue()));
						} else
						if (attributes.item(i).getNodeName() == "y") {
							lastAtom.setY(Double.parseDouble(attributes.item(i).getNodeValue()));
						}
						if (attributes.item(i).getNodeName() == "z") {
							lastAtom.setZ(Double.parseDouble(attributes.item(i).getNodeValue()));
						}
						
					}
				}
				
							
				
				System.out.println("atom done");
				
				
				atomAnzahl++;
			} catch (Exception e) {
				e.printStackTrace();
			}
		}

	
		
		if(node.getNodeName() == "ww"){
			WWManager ww = WWManager.getInstance();
			int jj =0;
			float wwv = 0;
			if (node.hasAttributes()) {
				NamedNodeMap attributes = node.getAttributes();
				for (int i=0; i < attributes.getLength(); i++) {
					if(attributes.item(i).getNodeName().toLowerCase() == "number")
						jj = Integer.parseInt(attributes.item(i).getNodeValue());
					if(attributes.item(i).getNodeName().toLowerCase() == "value")
						wwv = Float.parseFloat(attributes.item(i).getNodeValue());
				}
			}
			ww.setWW(jj, wwv);
		}
		
		if(node.getNodeName() == "mol"){
			//defaults setzen
			scale = 1.0f;
			color = "#999999";
			boolean hasPos = false;
			
			
			//attribute
			
			if (node.hasAttributes()) {
				
				NamedNodeMap attributes = node.getAttributes();
				for (int i=0; i < attributes.getLength(); i++) {
					if(attributes.item(i).getNodeName().toLowerCase() == "name"){
						lastClass = attributes.item(i).getNodeValue();
						
					} else
					
					if(attributes.item(i).getNodeName().toLowerCase() == "color"){
						color=attributes.item(i).getNodeValue();
						System.out.println(color);
					} else
						
					if(attributes.item(i).getNodeName().toLowerCase() == "xyz"){
						setXYZ(attributes.item(i).getNodeValue());
						hasPos = true;
					} else
					if(attributes.item(i).getNodeName().toLowerCase() == "scale"){
						scale = Float.parseFloat((attributes.item(i).getNodeValue()));
					}
				}
			}
			
			
			//erstellung
			
			Stack<AtomNeighbours> tmp = new Stack<AtomNeighbours>();
			try {
				if (!atombuf.isEmpty())
					tmp.addAll(atombuf);
				if (slist == null)
					slist = new ArrayList<Structure>();
				
				lastClass=lastClass.toLowerCase();
				
				if(lastClass.equals("dodekaeder"))
					slist.add(new Dodekaeder(tmp));
				else if (lastClass.equals("surface"))
					slist.add(new Surface(tmp));
				else 
					slist.add(new Structure(tmp));
				
				if (!slist.get(slist.size()-1).setColor(color))
					System.exit(1);
				
				// fix if no position is given by XML
				if(hasPos){
					slist.get(slist.size()-1).setX(lastAtom.getX());
					slist.get(slist.size()-1).setY(lastAtom.getY());
					slist.get(slist.size()-1).setZ(lastAtom.getZ());
				}
				
				slist.get(slist.size()-1).setScale(scale);
				
				atombuf.clear();
				atomAnzahl = 0;
			} catch (Exception e) {
				e.printStackTrace();
			}
			

						
		}

		if (node.getNodeName() == "workspace"){
			if (node.hasAttributes()) {
				NamedNodeMap attributes = node.getAttributes();
				for (int i=0; i < attributes.getLength(); i++) {
					if(attributes.item(i).getNodeName().toLowerCase() == "xyz"){
						workspaceChanged = true;
						//TODO write me
					}
					if(attributes.item(i).getNodeName().toLowerCase() == "bgcolor"){
						workspaceChanged = true;
						String colorString = attributes.item(i).getNodeValue();
						if(colorString.length() == 7 && colorString.startsWith("#")){
							
							background[0] = (Integer.parseInt(colorString.substring(1, 3), 16) / 255);
							background[1] = (Integer.parseInt(colorString.substring(3, 5),16) / 255);
							background[2] = (Integer.parseInt(colorString.substring(5, 7),16) / 255);
						}
						
					}
				}
			}
		}
		
	
		
		
		buf += "</" + node.getNodeName() + ">\n";
		return buf;
	}

	
	

	
	/**
	 * Second parse which buildes the connections between each Structure / Atom
	 * @param node
	 * @return reinterpretated XML file
	 * @see model.FileXML#Parse(Node)
	 */
	protected String secondParse(Node node){
		if(node.getNodeName() == "mol"){
			molAnzahl++;
			//TODO  remove me ... 
			atomAnzahl = 0;
		}
		
		if(node.getNodeName() == "atom"){
			atomAnzahl++;
		}
		
		if(node.getNodeName() == "connect"){
			Stack<AtomNeighbours> buffer = new Stack<AtomNeighbours>();
			for (Structure s: slist) {
				buffer.addAll(s.getAtome());
								
			}
			
			AtomNeighbours aNow = slist.get(molAnzahl-1).getAtom(atomAnzahl-1);			
			AtomNeighbours neigh = null;
			if(node.hasAttributes()){
				NamedNodeMap attributes = node.getAttributes();
				for (int i = 0; i < attributes.getLength(); i++) {
					if (attributes.item(i).getNodeName() == "nn") {
						//NOTE when using the faked first atom -> this has to be without -1
						
						String val = attributes.item(i).getNodeValue();
						if(!val.contains("@")){
							neigh = buffer.get(Integer.parseInt(val)-1);
						} else {
							int size = 0;
							String[] split = {"",""};
							split = val.split("@",2);
							int end = Integer.parseInt(split[1])-1;
							for(int o=0; o < slist.size() && o < end; o++){
								Structure s = slist.get(o);
								size += s.getAtome().size();	
							}
							size += (Integer.parseInt(split[0])-1);
							neigh = buffer.get(size);
						}
						
						WWManager ww = WWManager.getInstance();
						
						aNow.setNeighbours(neigh);
						
					}
					
					
				}
				
				for (int i = 0; i < attributes.getLength(); i++) {
					if (attributes.item(i).getNodeName() == "jj") {
						WWManager ww = WWManager.getInstance();
						
						int jj = Integer.parseInt(attributes.item(i).getNodeValue());
						ww.addJJ(jj, neigh);
						
						if(neigh != null){
							aNow.setNeighbourJJ(neigh, jj);
						}
					}
				}
				
				
				
				
			}
			
			return (molAnzahl -1)+"."+(atomAnzahl-1) +" " + node.getNodeName() +" " + node.getNodeValue();
			
			
		}
		
		
		String buf = new String();
		NodeList subNodes = node.getChildNodes();
		for (int i = 0; i < subNodes.getLength(); i++) {
			Node subNode = subNodes.item(i);
			if (subNode.getNodeName().equals("#text")) {
				if (subNode.getNodeValue().trim().equals(""))
					buf += "\n\t" + secondParse(subNode);
				else {
					System.out.println("+++++++" + subNode.getNodeValue());
				}
			} else {
				buf += "\n\t" + secondParse(subNode);
			} 
			
		}
		return buf;
	}
	
	/**
	 * Start Parsing on a specific startelement 
	 * @param name	TAG name where to start
	 * @return	XML in a reinterpreted form without higher elements
	 */
	public String getElementByName(String name) {
		return getElement(doc.getElementsByTagName(name).item(0));
	}

	/**
	 * Startup method for XML interpretations
	 * @return	XML in new form
	 */
	public String getAllElements() {
	
		System.out.println(getElement(doc.getFirstChild()));
	
		return "";
	}
	
	
		
	
	/*
	 * 
	 * interpreter teil
	 * 
	 * 
	 */
	
	/**
	 * converts a ";" seperated String into values for coords
	 * @param xyz values in seperated form
	 */
	private void setXYZ(String xyz){
		String test[]=xyz.split(";");
		lastAtom.setX(Double.parseDouble(test[0]));
		lastAtom.setY(Double.parseDouble(test[1]));
		lastAtom.setZ(Double.parseDouble(test[2]));
	}
	
	public boolean workspaceSetting(){
		return workspaceChanged;
	}
	
	/**
	 * Gettermethod for Backgroundcolor
	 * @return GL styled Color array
	 */
	public float[] getBackground(){
		return background;
	}
	
	/**
	 * TODO build a camPos object
	 * @return	the Cammeras Possition in space
	 */
	public Object3D getCamPos(){
		return camPos;
	}
	
}
