package objects;

import java.util.Stack;

import javax.media.opengl.GL;
import javax.media.opengl.glu.GLU;

import java.awt.Color;

/**
 * 
 * This class should be used like a interface but can be (whenever there is no Class for this kind
 * of data yet) used as alternativ.
 * Attention!
 * Whenever possible try to extend this class and not to use this class as a real 
 * Object. Its just the Basic setup!
 *
 */

public class Structure extends Objekt {

	Stack<AtomNeighbours> atome;
	
	static float silver[] = { 0.9f, 0.9f, 0.9f, 1.0f };
	static float green[] = { 0.0f, 0.8f, 0.2f, 1.0f };
	static float blue[] = { 0.0f, 0.2f, 0.8f, 1.0f };
	static float red[] = { 0.8f, 0.1f, 0.0f, 1.0f };
	static protected GLU glu = new GLU();
	static protected GL gl;
	protected int glListed = 0;

	protected AtomNeighbours lastSelected = null; 
	
	protected boolean drawConnection = false;
	protected boolean debug = false;
	
	private float color[] = {.8f,.8f,.8f,1f};
	
	protected boolean init = false;
	
	
	
	/**
	 * the tolerance is importent for the automated neighbour description.
	 * if you increase the tolerance you can get multiple atoms.
	 * 
	 * the toleranc has to be greater then 0 because the ensure calculation failors of
	 * sqrt
	 * 
	 */
	private double toleranc = 0.01d;
	
	/**
	 * Constructor which already have Atoms
	 * @param atome
	 */
	public Structure(Stack<AtomNeighbours> atome) {
		super();
		
		this.atome = atome;
	}
	/**
	 * Constructor which force to generate the belonging Atoms 
	 *
	 */
	public Structure() {
		super();
		
		atome = new Stack<AtomNeighbours>();
		generate();
	
	}

	/**
	 * 
	 * @param _drawConnection	Draw a connection between Neighbours
	 */
	public Structure(boolean _drawConnection){
		drawConnection = _drawConnection;
		
		atome = new Stack<AtomNeighbours>();
		generate();
	
	}
	
	/**
	 * 
	 * @param _drawConnection	Draw a connection between Neighbours
	 * @param atome		Atoms which are already defined
	 */
	public Structure(boolean _drawConnection, Stack<AtomNeighbours> atome){
		this.atome = atome;
	}
	
	/**
	 * ATTENTION:
	 * This method have to be overloaded.
	 * It will be called on startup and has no effect in this class.
	 * In the extend class this should generate the Atommodel or start the loading
	 * process!
	 * 
	 */
	protected void generate() {
		if (debug) System.err.println("overload me");
		
	}

	/**
	 * Getter for last selected Atom (importent for click handling)
	 * @return the lastSelected	The last selected Atom
	 */
	public AtomNeighbours getLastSelected() {
		return lastSelected;
	}

	/**
	 * Setter Method for the last selected Atom
	 * @param lastSelected the last selected to set
	 */
	public void setLastSelected(AtomNeighbours lastSelected) {
		this.lastSelected = lastSelected;
	}

	/**
	 * 
	 * @return the atome Stack
	 */
	public Stack<AtomNeighbours> getAtome() {
		return atome;
	}

	/**
	 * 
	 * will be executed in the openGL context
	 * 
	 */
	protected void glgenerate(){}
	
	/**
	 * Precompiles the Object 
	 * If you want to have realtime interaction youve got to overload this method.
	 * The same method will be used in Atom
	 * @see model.Atom#init()
	 * 
	 * This method should never be called directly!
	 * @see model.Structure.reinit()
	 *
	 */
	private void init() {

		gl = glu.getCurrentGL();

		glListed = gl.glGenLists(1);
		//from GL.GL_COMPILE_AND_EXECUTE to 
		gl.glNewList(glListed, GL.GL_COMPILE);
		//to prevent flickering
		
		//execute GL related content
		glgenerate();
	
		for (AtomNeighbours atom : atome) {
			if(atom != null){
				if (atom.isSelected()) gl.glMaterialfv(GL.GL_FRONT, GL.GL_AMBIENT_AND_DIFFUSE, red, 0);
				else gl.glMaterialfv(GL.GL_FRONT, GL.GL_AMBIENT_AND_DIFFUSE, color, 0);
				if(this.getScale() == 1.0f)						
					atom.draw();
				else 
					atom.draw(this.getScale());
			}
			if (drawConnection && !atom.getNeighbours().isEmpty()){
				gl.glPushMatrix();
	
				gl.glBegin(GL.GL_LINES);
	
					//TODO rewrite this, todo it better 
				
				
				
					for (AtomNeighbours neighbours : atom.getNeighbours()) {
						//einschränkung auf nur Mol basierte Atome
						if(this.atome.contains(neighbours)){
							gl.glVertex3d(atom.x*this.getScale(), atom.y*this.getScale(), atom.z*this.getScale());
							gl.glVertex3d(neighbours.x*this.getScale(), neighbours.y*this.getScale(), neighbours.z*this.getScale());
							gl.glNormal3d((atom.x*this.getScale() + neighbours.x*this.getScale()) / 2,
									(atom.y*this.getScale() + neighbours.y*this.getScale()) / 2,
									(atom.z*this.getScale() + neighbours.z*this.getScale()) / 2);
						}
					}
				gl.glEnd();
				gl.glPopMatrix();
			}

		}

		gl.glEndList();

	
	}

	/**
	 * Set the Atom Stack by hand. This method can be used after you already used the Constructor for a standard Object
	 * @param atome
	 *            the atome to set
	 */
	public void setAtome(Stack<AtomNeighbours> atome) {
		this.atome = atome;
		init();
	}

	/**
	 * Set the color for a whole Molecule 
	 * @param c	Color as Floating array. values can be between 0 and 1.0
	 * @return	true
	 */
	public boolean setColor(float[] c){
		color[0] = c[0];
		color[1] = c[1];
		color[2] = c[2];
		color[3] = c[3];
		return true;
	}
	
	/**
	 * Sets the Color by HTML styled String
	 * @param colorString	Color have to start with # and be 6 times a HEX value (0-F)
	 * @return	true
	 */
	public boolean setColor(String colorString){
		if(colorString.length() != 7 && colorString.startsWith("#"))
			return false;
		float r = ((float) (Integer.parseInt(colorString.substring(1, 3), 16)) / 255f);
		float g = ((float) (Integer.parseInt(colorString.substring(3, 5),16)) / 255f);
		float b = ((float) (Integer.parseInt(colorString.substring(5, 7),16)) / 255f);
		
		color[0] = r;
		color[1] = g;
		color[2] = b;
		
		
		return true;
	}
	
	/**
	 * Sets the Color by Java Color object
	 * @param c	
	 * @return
	 * @see java.awt.Color
	 */
	public boolean setColor(Color c){
		try{
		color[0] = c.getRed() /255f;
		color[1] = c.getGreen() / 255f;
		color[2] = c.getBlue() /255f;
		return true;
		} catch (Exception e){
			e.printStackTrace();
			return false;
		}
	}
	/**
	 * 
	 * @return Java Color object which representates the Color	
	 * @see java.awt.Color
	 */
	public Color getColor(){
		return new Color(color[0], color[1], color[2]);
	}
	
	/**
	 * This method checks about the init state of a object
	 * init should never be called directly!
	 *
	 */
	protected void isInit(){
		
		if (!init){
			if(gl == null)
				init();
			gl.glDeleteLists(glListed, 1);
			init();
		}
		init = true;
	}
	
	/**
	 * Handles the Translation directly
	 * Just a way to access the translation without any knowledge of the correct GL code
	 * @param gl
	 */
	public void translate(GL gl){
		gl.glTranslatef((float) x, (float) y, (float) z);
	}
	
	/**
	 * Core Method to draw the Object
	 *
	 */
	public void draw() {
		isInit();
		gl = glu.getCurrentGL();
		gl.glPushMatrix();

			translate(gl);
			gl.glMaterialfv(GL.GL_FRONT, GL.GL_AMBIENT_AND_DIFFUSE, color, 0);
			gl.glCallList(glListed);
			
		gl.glPopMatrix();
		
	}
	
	/**
	 * 
	 * @param AtomID	Atomnumber in the Structure of this object
	 * @return	Atom out of the Structure
	 */
	public AtomNeighbours getAtom ( int AtomID){
	    	return atome.get(AtomID);
	} 
	   
	/**
	 * Whenever you change values which change the look of objects you need to 
	 * use this function to recompile the Objects
	 *
	 */ 
	public void reinit() {
		
		
		init = false;
	}
	
	/**
	 *	XML format of all information about the deeper objects and itself 
	 * @see core.RenderList#toXML()
	 */
	
	public String XMLAttribute(){
		//TODO minimalisieren. Unnütze Daten, durch abfragen, entfernen
		return " name=\"" + this+ "\" " +super.XMLAttribute() + " color=\"#"+
			convertColor(color[0])+
			convertColor(color[1])+
			convertColor(color[2])+
			"\"";
	}
	
	/**
	 * Converts a GL color (part) to a Hex value
	 * @param c		GL color part
	 * @return		hex equivalent
	 */
	private String convertColor(float c){
		String tmp = Integer.toHexString(Math.round(255*c));
		if(tmp.length() == 1)
			tmp = '0'+tmp;
		
		return tmp;
		
	}
	
	/**
	 * Simple names for Objects
	 */
	public String toString(){
		return this.getClass().getSimpleName();
	}
	
	/**
	 * Gettermethod for the Option: Draw Connection between Neighbours
	 * @return the drawConnection
	 */
	public boolean isDrawConnection() {
		return drawConnection;
	}

	/**
	 * Set the draw Connection and reinitialize the object
	 * @param drawConnection the drawConnection to set
	 */
	public void setDrawConnection(boolean drawConnection) {
		this.drawConnection = drawConnection;
		reinit();
	}
	
	/**
	 * Gets the Neighbours by radius. Only works for one mol!
	 * Watch the "toleranc" Class Variable which allows you to increase the range of atoms.
	 */
	public void autoCompletNextNeighbours(){
		
		Stack<AtomNeighbours> neighBuf = new Stack<AtomNeighbours>();
		
		//für alle atome nachbarn definieren
		for (AtomNeighbours a : atome) {
			//nachbarn suchen
			
			neighBuf.clear();
			double sum = 1000000000.0d;
			for (AtomNeighbours possibleN : atome) {
				
				if(possibleN != a){
					double buf = a.diff(possibleN);
					if(sum == buf || ((sum + toleranc) > buf  && (sum - toleranc) < buf))
						neighBuf.add(possibleN);
					else
					if (buf < sum){
						sum = buf;
						neighBuf.clear();
						neighBuf.add(possibleN);			
					}
						
						
					
				}
					
			}
			for(AtomNeighbours found: neighBuf){
				WWManager wwman = WWManager.getInstance();
				a.setNeighbourJJ(found, 99);
				
				wwman.addJJ(99, a);
				a.setNeighbours(found);
				
			}
		}
	}
	

}
