/**
 * 
 */
package objects.nononjects;

import java.util.Stack;
import objects.*;
/**
 * 
 * Defines a transaction between Atoms.
 * A Transaction knows every connected atom
 *
 */
public class WechselWirkung {

	/**
	 * 
	 */
	private int identifier;
	private float value;
	private Stack<Atom> connected = null;
	/**
	 * Constructor for default values.
	 *
	 */
	public WechselWirkung() {
		init();
	}
	
	/**
	 * Constructor with specified parameters.
	 * @param id	transaction index 
	 * @param val	transaction value
	 */
	public WechselWirkung(int id, float val){
		setIdentifier(id);
		setValue(val);
		init();
	}
	
	
	
	private void init(){
		connected = new Stack<Atom>();
	}

	/**
	 * 
	 * @return the connected atoms which belong to this transaction
	 */
	public Stack<Atom> getConnected() {
		return connected;
	}

	/**
	 * Connect more atoms at once.
	 * @param connected the connected to set
	 */
	public void setConnected(Stack<Atom> connected) {
		this.connected = connected;
	}

	/**
	 * Adds a simple connection to this transaction.
	 * @param a	Atom to connect
	 */
	public void addConnection(Atom a){
		System.out.println("connected one");
		
		connected.add(a);
	}
	
	/**
	 * The indentifier representates the transaction index.
	 * @return the identifier
	 */
	public int getIdentifier() {
		return identifier;
	}

	/**
	 * Manual override of the transaction identifier.
	 * Make sure you care about the WWManager.
	 * @see objects.WWManager
	 * @param identifier the identifier to set
	 */
	public void setIdentifier(int identifier) {
		this.identifier = identifier;
	}

	/**
	 * 
	 * @return the core Transaction value
	 */
	public float getValue() {
		return value;
	}

	/**
	 * @param value the Transaction value to set
	 */
	public void setValue(float value) {
		this.value = value;
	}
	
	/**
	 * Simplified transaction view. 
	 * This is important for the JTree.
	 */
	public String toString(){
		return identifier + " => " + value;
	}

}
